import pyodbc

class OrmManager(object):
    def __get__(self, instance, owner):
        self.owner = owner
        self.connection = pyodbc.connect(
            driver="{SQL Server}",
            database="PandaBOS",
            server="LAPTOP-K3C6JQ36\\MSSQLPANDAPOS",
            user="sa",
            password="esat3535",
        )
        return self

    def create(self, **kwargs):
        cursor = self.connection.cursor()
        
        sql = f"""
        INSERT INTO {self.owner} ({', '.join(list(kwargs.keys()))}) VALUES ({', '.join(['?']*len(kwargs))})
        """
        
        
        cursor.execute(sql, list(kwargs.values()))
        cursor.commit()
        return self.owner(**kwargs)
    
    def all(self):
        cursor = self.connection.cursor()
        sql = f"SELECT * FROM {self.owner.__name__}"
        cursor.execute(sql)
        columns = [column[0] for column in cursor.description]
        rows = [self.owner(**dict(zip(columns, row))) for row in cursor.fetchall()] #cursor.fetchall()
        return rows
    
    
    def where_clause_generate(self, **kwargs):
        where_clause = 'WHERE ' + ' AND '.join([k +' = ?' for k in kwargs.keys()])
        return where_clause
    
    
    def filter(self, **kwargs):
        cursor = self.connection.cursor()
        sql = f"""
        SELECT * FROM {self.owner.__name__} {self.where_clause_generate(**kwargs)}"""
        cursor.execute(sql, list(kwargs.values()))
        columns = [column[0] for column in cursor.description]
        rows = [self.owner(**dict(zip(columns, row))) for row in cursor.fetchall()] #cursor.fetchall()
        return rows


import datetime

from Metrik.models.ORMModel import Model


class Orders(Model):
    Id: int
    TicketId: int
    WarehouseId: int
    DepartmentId: int
    TerminalId: int
    MenuItemId: int
    MenuItemName: str
    PortionName: str
    Price: float
    Quantity: float
    PortionCount: int
    Locked: bool
    CalculatePrice: bool
    DecreaseInventory: bool
    IncreaseInventory: bool
    OrderNumber: int
    CreatingUserName: str
    CreatedDateTime: datetime
    LastUpdateTime: datetime
    AccountTransactionTypeId: int
    ProductTimerValueId: int
    GroupTagName: str
    GroupTagFormat: str
    Separator: str
    PriceTag: str
    Tag: str
    DisablePortionSelection: bool
    OrderUid: str
    Taxes: str
    OrderTags: str
    OrderStates: str
    

class Tickets(Model):
    Id: int
    LastUpdateTime: datetime.datetime
    TicketVersion: datetime.datetime
    TicketUid: str
    TicketNumber: str
    Date: datetime.datetime
    LastOrderDate: datetime.datetime
    LastPaymentDate: datetime.datetime
    PreOrder: bool
    IsClosed: bool
    IsLocked: bool
    RemainingAmount: float
    TotalAmount: float
    DepartmentId: int
    TerminalId: int
    TicketTypeId: int
    Note: str
    LastModifiedUserName: str
    TicketTags: str
    TicketStates: str
    TicketLogs: str
    LineSeparators: str
    ExchangeRate: float
    TaxIncluded: bool
    Name: str
    TransactionDocument_Id: int
    IsOpened: bool
    TotalAmountPreTax: float
    orders: list[Orders]
    Status: int
    



from .OrmManager import OrmManager
from typing import get_type_hints, get_args
import datetime

class Model(object):
    objects = OrmManager()
    def __init__(self, **kwargs) -> None:
        self.__dict__.update(kwargs)
        hints = get_type_hints(self)
        # Sonra tarih tipindekileri dönüştürün
        for key, hint_type in hints.items():
            if hint_type == datetime.datetime:
                attr_value = getattr(self, key, None)  # Varsayılan olarak None dön
                if attr_value and isinstance(attr_value, str):  # Değer string ve atanmışsa
                    try:
                        # Tarih formatı kontrolü ve dönüştürme
                        setattr(self, key, datetime.datetime.fromisoformat(attr_value))
                    except ValueError:
                        # Geçersiz tarih formatı, uygun bir hata yönetimi ekleyin
                        pass
            



import pyodbc

from Metrik.models.Ticket import Orders, Tickets

class SQLHelper:
    def __init__(self, server: str, port: str, db_name: str, driver: str, user: str = "", password: str = ""):
        self.server = server
        self.port = port
        self.db_name = db_name
        self.driver = driver
        self.user = user
        self.password = password
        
        self.connection = pyodbc.connect(
            driver=self.driver,
            database=self.db_name,
            server=self.server,
            user=user,
            password=self.password,
        )
    
    
    def get_orders(self):
        orders = Orders.objects.all()
        
        return orders
    
    def get_orders_by_ticket_id(self, ticket_id):
        orders = Orders.objects.filter(TicketId=ticket_id)
        
        return orders
    
    
    def get_tickets(self) -> list[Tickets]:
        tickets: list[Tickets] = Tickets.objects.all()
        for ticket in tickets:
            ticket.orders = self.get_orders_by_ticket_id(ticket.Id)
            
        return tickets
    
    def get_tickets_by_status(self, status: int) -> list[Tickets]:
        tickets: list[Tickets] = Tickets.objects.filter(Status=status)
        for ticket in tickets:
            ticket.orders = self.get_orders_by_ticket_id(ticket.Id)
        return tickets
    
        
        



from setuptools import find_packages, setup

setup(
    name="PandaPosMetrik",
    version="1.8",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "pyodbc",
    ]
)
